function Denon(DeviceName)                 // properties: "DeviceName" - device name in the project 
{   
   this.DeviceName   = DeviceName;                     // device name in the project  
   this.DEVICE       = IR.GetDevice(this.DeviceName);  // identifier of the device instance in the system     
   
   // initialize variables
   IR.SetVariable("Drivers."+DeviceName+".Online Status",0);  
   IR.SetVariable("Drivers."+DeviceName+".Mute Status",0); 
   IR.SetVariable("Drivers."+DeviceName+".Power",0); 
   IR.SetVariable("Drivers."+DeviceName+".Parameter",0);    
   IR.SetVariable("Drivers."+DeviceName+".Select Input Source",0);
   IR.SetVariable("Drivers."+DeviceName+".Tuner CH",0);
   IR.SetVariable("Drivers."+DeviceName+".Volume",0);
   
   function getStatus3()
    {
     THIS.DEVICE.Send(["MS?\r"]);
    }   
   function getStatus2()
    {
     THIS.DEVICE.Send(["MV?\r"]);
     IR.SetTimeout(1000,getStatus3);
    } 
   function getStatus1()
    {
     THIS.DEVICE.Send(["SI?\r"]);          
     IR.SetTimeout(1000,getStatus2);
    }
   function getStatus()
    { 
     IR.Log("start retrieving status");
     THIS.DEVICE.Send(["PW?\r"]);   
     IR.SetTimeout(1000,getStatus1);
    }
       
   // Event: connection is established
   IR.AddListener(IR.EVENT_ONLINE, this.DEVICE, function()
   {    
      IR.Log("ONLINE DENON")
      IR.SetVariable("Drivers."+DeviceName+".Online Status",1);    
      IR.SetTimeout(1000,getStatus);
   }, this);
   
   // Event: connection is not established
   IR.AddListener(IR.EVENT_OFFLINE, this.DEVICE, function()
   {    
      IR.Log("OFFLINE DENON")
      IR.SetVariable("Drivers."+DeviceName+".Online Status",0); 
   }, this);
   
   
   // Event: response from the device is received 
   IR.AddListener(IR.EVENT_RECEIVE_TEXT, this.DEVICE, function(data)
   {  
      // variables for parsing
      var command = "";
      var parameter = "";
      var length = "";
      var buf;
  
      // identification command
      command = data.substring( 0, 2 ); 
      switch(command)
      {
         case "PW":    // command: "Power"         
            parameter = data.substring( 2, data.length-1 );
            if(parameter == "ON")
            {
               IR.SetVariable("Drivers."+DeviceName+".Power",1);                  
            }
            else if(parameter == "STANDBY")
            {
               IR.SetVariable("Drivers."+DeviceName+".Power",0);    
            }            
            break;
            
         case "MV":     // command: "MASTER VOLUME"
            var voltemp = data.split('\r');
            for(i=0;i<voltemp.length-1; i++)
             {
              var x = voltemp[i].toString();   
              if( x.indexOf('MVMAX') ==-1)
               {
                var xval = x.match(/\d{1,2}/);
                IR.SetVariable("Drivers."+DeviceName+".Volume",xval);
               }
             }
            break;

         case "MU":     // command: "VOLUME MUTE"
            parameter = data.substring( 2, data.length-1 );   
            
            if(parameter == "ON")
            {
               IR.SetVariable("Drivers."+DeviceName+".Mute Status",1);                  
            }
            else if(parameter == "OFF")
            {
               IR.SetVariable("Drivers."+DeviceName+".Mute Status",0);    
            } 
            break; 
          case "SI":     // command: "Select INPUT source"
            var source  = ['PHONO','CD','TUNER','DVD','BD','TV','SAT/CBL','MPLAY','GAME','AUX1','AUX2','HDRADIO','NET','PANDORA','SIRIUSXM','LASTFM','FLICKR','FAVORITES','IRADIO','SERVER','USB/IPOD','USB','IPD','IRP','FVP']; 
            parameter = data.substring( 2, data.length-1 );
            parameter = THIS.FindToken(parameter,source);          
            if ( parameter > -1 )
            {
                IR.SetVariable("Drivers."+DeviceName+".Select Input Source",parameter); 
             if( parameter == 1 )
               IR.SetVariable("Global.Source","SONOS");
              else  
               IR.SetVariable("Global.Source",source[parameter]); 
            }            
            break;    
          case "TF":     // command: "Analog tuner Frequency"
            parameter = data.substring( 2, 4 );
            if(parameter == "AN")
            {
               parameter = data.substring( 4, data.length-1 );                   
               parameter = (parseFloat(parameter)/100).toFixed(2);
               parameter = parameter.toString();
               IR.SetVariable("Drivers."+DeviceName+".Tuner Frequency",parameter);  
             }
             break;  
          case "MS":
            var surround = ['MOVIE','MUSIC','GAME','DIRECT','PURE DIRECT','STEREO','STANDARD','DOLBY DIGITAL','DTS SURROUND','MCH STEREO','ROCK ARENA','JAZZ CLUB','MONO MOVIE','MATRIX','VIDEO GAME','VIRTUAL'];
            parameter = data.substring( 2, data.length-1 );
            parameter = THIS.FindToken(parameter,surround); 
            if ( parameter > -1 )
            {
             IR.SetVariable("Drivers."+DeviceName+".Surround",parameter);            
             IR.SetVariable("Global.Surround",surround[parameter]); 
            }
            break;                         
          case "TP":     // command: "Analog tuner channel"
            parameter = data.substring( 2, 4 );
            if(parameter == "AN")
            {
               parameter = data.substring( 4, 5 );
               buf = -1;
                    if(parameter == 'A') { buf = 0; }
               else if(parameter == 'B') { buf = 1; }
               else if(parameter == 'C') { buf = 2; }
               else if(parameter == 'D') { buf = 3; }
               else if(parameter == 'E') { buf = 4; }
               else if(parameter == 'F') { buf = 5; }
               else if(parameter == 'G') { buf = 6; }
               
               if(buf > -1)
               {
                  parameter = data.substring( 5, 6 );
                  parameter = parseInt(parameter);
                  if(parameter>-1 && parameter <10)
                  {
                     parameter = buf*10 + parameter;                                        
                     IR.SetVariable("Drivers."+DeviceName+".Tuner CH",parameter);                  
                  } 
               }
            }
            break; 
          case "SY":     // command: "Front panel lock state"
            parameter = data.substring( 2, 4 );
            if(parameter == "SYREMOTE LOCK ON")
            {
               IR.SetVariable("Drivers."+DeviceName+".Panel Lock",1);  
            }
            else if(parameter == "SYREMOTE LOCK OFF")
            {
               IR.SetVariable("Drivers."+DeviceName+".Panel Lock",0);  
            }                                                                
            break;     
      }
   }, this); 
    
    
   // function to convert the values ??in the specified range (from 0 to 100 in from min to max) and the formation of lines
   // parameters:
   // min - the minimum value of the range
   // max - the maximum value of the range
   // n   - number of characters
   // val - initial value to be converted
   this.ValToStr = function (min, max, n, val)  
   {  
      var str = "";    
      val = Math.round((max-min)*val/100+min);
      str = "" + val;      
      while(str.length<n)
      {
         str = "0" + str;
      }      
      return str;
   }
   
   
   // function to check the item "name" in the array "arr"
   this.FindToken = function(name,arr)
   {
      var i;
      for(i=0;i<arr.length;i++)
      {
         if(name == arr[i])
            return i;
      }
      return -1;
   }
   
   // Event: sending the command to the device   
   var THIS = this;
   IR.AddListener(IR.EVENT_CHANNEL_SET, IR.GetDevice(this.DeviceName), function(name, value)
   {
      var parameter;
      var buf, str;
      
      // search for instruction identifier     
      switch(name)
      {
         case "Set Volume (p)":           
            THIS.DEVICE.Send(["MV" + THIS.ValToStr(0,98,2,value) + "\r"]);
            break;
            
         case "Select Input (p)":     
            if (THIS.FindToken(value,['PHONO','CD','TUNER','DVD','BD','TV','SAT/CBL','MPLAY','GAME','AUX1','AUX2','HDRADIO','NET','PANDORA','SIRIUSXM','LASTFM','FLICKR','FAVORITES','IRADIO','SERVER','USB/IPOD','USB','IPD','IRP','FVP'])>-1)            
            {
               THIS.DEVICE.Send(["SI" + value + "\r"]);
            }
            break;
            
         case "Power Toggle":         
            parameter = IR.GetVariable("Driver." + DeviceName + "." + "Power");        
            if(parameter == 1)
            {
               THIS.DEVICE.Send(["PWSTANDBY\r"]);                  
            }
            else if(parameter == 0)
            {
               THIS.DEVICE.Send(["PWON\r"]);    
            }                             
            break;   
             
         case "Mute Toggle":
            parameter = IR.GetVariable("Driver." + DeviceName + "." + "Mute Status");        
            if(parameter == 1)
            {
               THIS.DEVICE.Send(["MUOFF\r"]);                  
            }
            else if(parameter == 0)
            {
               THIS.DEVICE.Send(["MUON\r"]);    
            }         
            break;   
            
         case "Set Tuner Frequency (p)":
            str = IR.GetVariable("Driver." + DeviceName + "." + "Tuner Frequency");
            buf = parseFloat(str);
            buf = buf.toFixed(2)*100;            
            
            if(buf<1000000)
            {                
               str = buf.toString();             
               while(str.length<6)
               {
                  str = "0" + str;
               }          
               THIS.DEVICE.Send(["TFAN" + str + "\r"]);
            }            
            break;   
      }       
   });      
}

var Denon1 = new Denon("Denon1"); // object creation  ("device name")

IR.AddListener(IR.EVENT_ITEM_CHANGE, IR.GetItem("Main").GetItem("Tuner Frequency"), function()
{ 
   IR.SetVariable("Driver.Denon1.Tuner Frequency",IR.GetItem("Main").GetItem("Tuner Frequency").Text); 
});
